@@ -21,6 +21,15 @@ ENUMERATED_TABLES = {
     "PREDISPATCH": [("CONSTRAINT", 2), ("LOAD", 2)],
 }
 
+DEPRECATED_TABLES = {
+    "MTPASA": [
+        "CASESOLUTION",
+    ],
+}
+"""
+Deprecated tables
+"""
+
 DATETIME_FORMAT = "%Y/%m/%d %H:%M"
 """
 `nemseer` date format
@@ -48,6 +57,53 @@ FORECASTED_COL = {
 If it exists, `nemseer` uses the corresponding column for `forecasted` time filtering.
 """
 
+ID_COLS = {
+    "STUDYREGIONID",
+    "CONSTRAINTID",
+    "INTERCONNECTORID",
+    "DUID",
+    "CONNECTIONPOINTID",
+    "PARTICIPANTID",
+    "EXPORTGENCONID",
+    "IMPORTGENCONID",
+    "REGIONID",
+    "LINKID",
+    "USE_ITERATION_ID",
+    "RESERVELIMITID",
+    "SCENARIO",
+}
+
+TYPE_COLS = {
+    "BIDTYPE",
+    "RUNTYPE",
+    "RUN_NO",
+    "DEMAND_POE_TYPE",
+    "AGGREGATION_PERIOD",
+    "PERIOD_ENDING",
+    "EFFECTIVEDATE",
+    "VERSION_DATETIME",
+}
+
+DATETIME_COLS = {
+    "DATETIME",
+    "EFFECTIVEDATE",
+    "INTERVAL_DATETIME",
+    "RUN_DATETIME",
+    "AUTHORISEDDATE",
+    "LASTCHANGED",
+    "VERSION_DATETIME",
+    "DAY",
+    "PUBLISH_DATETIME",
+    "LATEST_OFFER_DATETIME",
+    "STARTDATE",
+    "ENDDATE",
+    "PERIOD_ENDING",
+    "GENCONID_EFFECTIVEDATE",
+    "BIDSETTLEMENTDATE",
+    "SETTLEMENTDATE",
+    "OFFERDATE",
+}
+
 INVALID_STUBS_FILE = ".invalid_aemo_files.txt"
 """File in :term:`raw_cache` that contains invalid/corrupted AEMO files"""
 
@@ -106,4 +162,3 @@ USER_AGENTS = [
         + "Raspbian Chromium/74.0.3729.157 Chrome/74.0.3729.157 Safari/537.36"
     ),
 ]
-"""List of user-agents"""
