@@ -1,11 +1,19 @@
 import logging
 from datetime import datetime
 from pathlib import Path
-from typing import Union
+from typing import List, Tuple, Union
 
 import pandas as pd
+import xarray as xr
 
-from .data import DATETIME_FORMAT, FORECASTED_COL, RUNTIME_COL
+from .data import (
+    DATETIME_COLS,
+    DATETIME_FORMAT,
+    FORECASTED_COL,
+    ID_COLS,
+    RUNTIME_COL,
+    TYPE_COLS,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -20,25 +28,7 @@ def _parse_datetime_cols(df: pd.DataFrame) -> pd.DataFrame:
         AEMO
         format
     """
-    dt_cols = {
-        "DATETIME",
-        "EFFECTIVEDATE",
-        "INTERVAL_DATETIME",
-        "RUN_DATETIME",
-        "AUTHORISEDDATE",
-        "LASTCHANGED",
-        "VERSION_DATETIME",
-        "DAY",
-        "PUBLISH_DATETIME",
-        "LATEST_OFFER_DATETIME",
-        "STARTDATE",
-        "ENDDATE",
-        "PERIOD_ENDING",
-        "GENCONID_EFFECTIVEDATE",
-        "BIDSETTLEMENTDATE",
-        "SETTLEMENTDATE",
-        "OFFERDATE",
-    }
+    dt_cols = DATETIME_COLS
     dt_cols_present = dt_cols.intersection(set(df.columns.tolist()))
     for col in dt_cols_present:
         df.loc[:, col] = pd.to_datetime(df[col], format=DATETIME_FORMAT)
@@ -100,16 +90,20 @@ def clean_forecast_csv(filepath_or_buffer: Union[str, Path]) -> pd.DataFrame:
     Warning:
         Removes duplicate rows. Raises a warning when doing so.
     """
-    # skip AEMO metadata
     df = pd.read_csv(filepath_or_buffer, skiprows=1, low_memory=False)
     # remove end of report line
     df = df.iloc[0:-1, :]
+    # skip AEMO metadata
     drop_cols = df.columns.tolist()[0:4]
     df = df.drop(drop_cols, axis="columns")
     df = _parse_datetime_cols(df)
     df = _parse_id_cols(df)
     if "PREDISPATCHSEQNO" in df.columns:
         df = _parse_predispatch_seq_no(df)
+    for col in [col for col in df.columns if df.dtypes[col] == "float64"]:
+        df[col] = pd.to_numeric(df[col], downcast="integer")
+    for col in [col for col in df.columns if df.dtypes[col] == "float64"]:
+        df[col] = pd.to_numeric(df[col], downcast="float")
     if any(dup_df := df.duplicated()):
         dup_rows = dup_df.loc[dup_df is True]
         logging.warning(
@@ -175,3 +169,90 @@ def apply_run_and_forecasted_time_filters(
         if col in df.columns:
             df = _filter_on_datetime_col(df, col, start, end)
     return df
+
+
+def to_xarray(df: pd.DataFrame, forecast_type: str):
+    """Converts a :class:`pandas.DataFrame` to a :class:`xarray.Dataset` using nemseer
+    definitions to determine Dataset dimensions.
+
+    The conversion is processed in chunks. If system memory usage exceeds 95%, the
+    conversion is terminated with a MemoryError. This is more informative than
+    the system killing the Python process.
+
+    Args:
+        df: pandas.DataFrame to be converted.
+        forecast_type: One of :data:`nemseer.forecast_types`.
+    Returns:
+        :class:`xarray.Dataset`.
+    Warning:
+        Raises a warning when attempting to convert high-dimensional data.
+    Raises:
+        MemoryError: If system memory utilisation exceeds 95% whilst converting
+            DataFrame chunks. Note that this error will not be raised if the memory
+            required to complete the Dataset merge exceeds available system memory.
+    """
+
+    def _determine_multiindex(
+        df: pd.DataFrame, forecast_type: str
+    ) -> Tuple[pd.MultiIndex, List[str]]:
+        """Ascertains which DataFrame columns are in nemseer's datetime, ID or
+        type column lists and then uses these columns to create a MultiIndex.
+
+        Args:
+            df: pandas DataFrame.
+            forecast_type: One of :data:`nemseer.forecast_types`.
+        Returns:
+            A MultiIndex that will be parsed as dimensions when converting to
+            :class:`xarray.Dataset`, and a list of the columns that the MultiIndex
+            incorporates.
+        """
+        multiindex_cols = []
+        names = []
+        for name, col in zip(
+            ("run_time", "forecasted_time"),
+            (RUNTIME_COL[forecast_type], FORECASTED_COL[forecast_type]),
+        ):
+            if col in df.columns:
+                names.append(name)
+                multiindex_cols.append(col)
+        for col in [col for col in df.columns if col not in multiindex_cols]:
+            if col in ID_COLS or col in TYPE_COLS:
+                names.append(col)
+                multiindex_cols.append(col)
+        multiindex = pd.MultiIndex.from_frame(df[multiindex_cols], names=names)
+        return multiindex, multiindex_cols
+
+    def _df_to_xarray(df: pd.DataFrame) -> xr.Dataset:
+        """Reformats supplied DataFrame to a MultiIndexed DataFrame and then
+        converts to :class:`xarray.Dataset`.
+
+        Args:
+            df: pandas DataFrame.
+        Returns:
+            DataFrame converted to xarray Dataset.
+        """
+        multiindex, multiindex_cols = _determine_multiindex(df, forecast_type)
+        df = df.set_index(multiindex)
+        df = df.drop(multiindex_cols, axis=1)
+        ds = df.to_xarray()
+        return ds  # type: ignore
+
+    dim_cols = [
+        col
+        for col in df.columns
+        if (
+            col in ID_COLS
+            or col in TYPE_COLS
+            or col in RUNTIME_COL[forecast_type]
+            or col in FORECASTED_COL[forecast_type]
+        )
+    ]
+    if len(dim_cols) >= 5 or any(
+        [col for col in df.columns if len(df[col].unique()) > 300]
+    ):
+        logging.warning(
+            "High-dimensional data. Large datetime requests may result in the Python "
+            + "process being killed by the system"
+        )
+    ds = _df_to_xarray(df)
+    return ds
